package control

import (
	"crypto/sha256"
	"encoding/hex"

	"git.frostfs.info/TrueCloudLab/frostfs-api-go/v2/rpc/client"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-cli/internal/commonflags"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-cli/internal/key"
	commonCmd "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/internal/common"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/services/control"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
	"github.com/spf13/cobra"
)

const (
	chainIDFlag    = "chain-id"
	chainIDHexFlag = "chain-id-hex"
)

var removeRuleCmd = &cobra.Command{
	Use:   "remove-rule",
	Short: "Remove local override",
	Long:  "Remove local APE override of the node",
	Run:   removeRule,
}

func removeRule(cmd *cobra.Command, _ []string) {
	pk := key.Get(cmd)

	var cnr cid.ID
	cidStr, _ := cmd.Flags().GetString(commonflags.CIDFlag)
	commonCmd.ExitOnErr(cmd, "can't decode container ID: %w", cnr.DecodeString(cidStr))

	rawCID := make([]byte, sha256.Size)
	cnr.Encode(rawCID)

	chainID, _ := cmd.Flags().GetString(chainIDFlag)
	hexEncoded, _ := cmd.Flags().GetBool(chainIDHexFlag)

	chainIDRaw := []byte(chainID)

	if hexEncoded {
		var err error
		chainIDRaw, err = hex.DecodeString(chainID)
		commonCmd.ExitOnErr(cmd, "can't decode chain ID as hex: %w", err)
	}

	req := &control.RemoveChainLocalOverrideRequest{
		Body: &control.RemoveChainLocalOverrideRequest_Body{
			Target: &control.ChainTarget{
				Name: cidStr,
				Type: control.ChainTarget_CONTAINER,
			},
			ChainId: chainIDRaw,
		},
	}

	signRequest(cmd, pk, req)

	cli := getClient(cmd, pk)

	var resp *control.RemoveChainLocalOverrideResponse
	var err error
	err = cli.ExecRaw(func(client *client.Client) error {
		resp, err = control.RemoveChainLocalOverride(client, req)
		return err
	})
	commonCmd.ExitOnErr(cmd, "rpc error: %w", err)

	verifyResponse(cmd, resp.GetSignature(), resp.GetBody())

	if resp.GetBody().GetRemoved() {
		cmd.Println("Rule has been removed.")
	} else {
		cmd.Println("Rule has not been removed.")
	}
}

func initControlRemoveRuleCmd() {
	initControlFlags(removeRuleCmd)

	ff := removeRuleCmd.Flags()
	ff.String(commonflags.CIDFlag, "", commonflags.CIDFlagUsage)
	ff.String(chainIDFlag, "", "Chain id")
	ff.Bool(chainIDHexFlag, false, "Flag to parse chain ID as hex")
}
